﻿local GetTime = GetTime

local debugMode = false

local GuildRosterTable, rosterCooldown, rosterCooldownStart, delayedCall


function GuildWho_Print(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg, 1, 1, 0)
end


-- debug message function, only visible in debug mode
function GuildWho_Debug(msg)
	if debugMode then
		DEFAULT_CHAT_FRAME:AddMessage("• "..msg, 1, 1, 0)
	end
end


-- function to toggle debug mode
function GuildWho_DebugMode()
	if not debugMode then
		debugMode = true
		GuildWho_Print("GuildWho: Debug mode ON.")
	else
		debugMode = false
		GuildWho_Print("GuildWho: Debug mode OFF.")
	end
end


function GuildWho_OnUpdate(self, elapsed)
	if GetTime() - rosterCooldownStart > 10 then
		GuildWho_Debug("Roster cooldown finished")
		rosterCooldown = false
		self:SetScript("OnUpdate", nil)
	end
end


function GuildWho_GetLookup(name, onlineCheck)
	local lookup, altChar, mainChar
	
	GuildWho_Debug(" ----- ")
	
	for character in pairs(GuildRosterTable) do
		-- does the provided search string match a guild member's name?
		if strlower(name) == strlower(character) then
			lookup = character
			GuildWho_Debug("lookup = "..lookup)
			
			-- search string matches an online character, so no need to look any further
			if GuildRosterTable[character]["online"] and onlineCheck then
				return lookup, character
			end
			
			-- if this character's officer note is EPGPs or blank it's the main, else it's the name of the main
			mainChar = GuildRosterTable[character]["note"] or character
			
			GuildWho_Debug("mainChar = "..mainChar)
			break
		end
	end
	
	if onlineCheck then
		for character in pairs(GuildRosterTable) do
			if (GuildRosterTable[character]["note"] == mainChar or character == mainChar) and GuildRosterTable[character]["online"] then
				altChar = character
				GuildWho_Debug("altChar = "..character)
				break
			end
		end
	end
	
	return lookup, altChar, mainChar
end


function GuildWho_UpdateGuildRoster()
	local numGuildies = GetNumGuildMembers(true)
	
	-- if custom guild roster is not yet initiated, and default guild roster has been loaded
	if not GuildRosterTable and	numGuildies > 0 then
		GuildWho_Debug(numGuildies.. " guild members.")
		-- initiate custom guild roster, get data from original guild roster
		GuildRosterTable = {}
	end
	
	GuildWho_Debug("Updating GuildRosterTable.")
	
	for i = 1, numGuildies do
		local name, _, _, _, _, _, publicnote, officernote, online = GetGuildRosterInfo(i)
		local note
		if GuildWhoData.note == "officer" then
			note = strmatch(officernote, "^(%a+)$")
		else
			note = strmatch(publicnote, "^(%a+)$")
		end
		GuildRosterTable[name] = {
			online = online,
			note = note,
		}
	end
end


function GuildWho_SlashHandlerWho(msg)
	if not msg or msg == "" then
		GuildWho_Print("Usage:")
		GuildWho_Print("'/gwho character' - find who character is online as")
		GuildWho_Print("'/gwho setnote' - toggle using public/officer note.")
		GuildWho_Print("'/alts character' - list all of character's alts")
		GuildWho_Print("'/gwho debug' - toggle debug messages.")
	elseif strlower(msg) == "debug" then
		GuildWho_DebugMode()
	elseif strlower(msg) == "setnote" then
		if GuildWhoData.note == "officer" then
			GuildWhoData.note = "public"
		else
			GuildWhoData.note = "officer"
		end
		GuildWho_Print("Now using "..GuildWhoData.note.." note.")
		GuildWho_UpdateGuildRoster()
	else
		if rosterCooldown or delayedCall then
			-- roster on cooldown, get data now
			local lookup, altChar = GuildWho_GetLookup(msg, true)
			
			GuildWho_Debug(" ----- ")
			GuildWho_Debug("Roster on cooldown, get data now")
			
			if not lookup then
				-- character wasn't a guild match
				GuildWho_Print("No character named "..strupper(strsub(msg, 1, 1))..strlower(strsub(msg, 2)).." found in guild.")
				return
			end
			
			-- print result
			if altChar then
				local asAlt
				-- make a link outta the character name
				if lookup ~= altChar then
					asAlt = " as ".."|Hplayer:"..altChar.."|h["..altChar.."]|h"
				else
					lookup = "|Hplayer:"..lookup.."|h["..lookup.."]|h"
					asAlt = ""
				end
				GuildWho_Print(lookup.." is online"..asAlt..".")
			else
				GuildWho_Print(lookup.." is offline.")
			end
		else
			-- roster cooldown is ready - delaying call
			GuildWho_Debug("Roster cooldown is ready - delaying call")
			delayedCall = {"Who", msg}
			GuildRoster()
		end
	end
end


function GuildWho_SlashHandlerAlts(msg)
	if not msg or msg == "" then
		GuildWho_Print("• How to use GuildWho:")
		GuildWho_Print("/gwho character - find who character is online as")
		GuildWho_Print("/gwho setnote - toggle using public/officer note.")
		GuildWho_Print("/alts character - list all of character's alts")
		GuildWho_Print("/gwho debug - toggle debug messages.")
	else
		if rosterCooldown or delayedCall then
			-- get main character of the given character
			local lookup, _, mainChar = GuildWho_GetLookup(msg)
			local hasAlts, header
			
			GuildWho_Debug(" ----- ")
			GuildWho_Debug("Roster on cooldown, get data now")
			
			if lookup then
				-- find alts of mainChar
				for character in pairs(GuildRosterTable) do
					if (GuildRosterTable[character]["note"] == mainChar or character == mainChar) and character ~= lookup then
						-- character is mainChar or an alt of mainChar, and not the searched character, so it's a match!
						hasAlts = true
						if not header then
							GuildWho_Print("Known alts of "..lookup..":")
							header = true
						end
						GuildWho_Print("  "..character)
					end
				end
				if not hasAlts then
					GuildWho_Print(lookup.." has no known alts.")
				end
			else
				-- searched character wasn't a guild match
				GuildWho_Print("No character named "..strupper(strsub(msg, 1, 1))..strlower(strsub(msg, 2)).." found in guild.")
			end
		else
			GuildWho_Debug("Roster cooldown is ready - delaying call")
			delayedCall = {"Alts", msg}
			GuildRoster()
		end
	end
end


SlashCmdList["GWHO_WHO"] = GuildWho_SlashHandlerWho
SLASH_GWHO_WHO1 = "/gwho"

SlashCmdList["GWHO_ALTS"] = GuildWho_SlashHandlerAlts
SLASH_GWHO_ALTS1 = "/alts"


hooksecurefunc("UnitPopup_OnClick", function()
	local dropdownFrame = _G[UIDROPDOWNMENU_INIT_MENU]
	local button = this.value
	local name = dropdownFrame.name
	
	if button == "GW_LISTALTS" then
		if name then
			if rosterCooldown then
				GuildWho_SlashHandlerAlts(name)
			else
				delayedCall = {"Alts", name}
				GuildRoster()
			end
		end
	end
end)


hooksecurefunc("UnitPopup_HideButtons", function()
	local dropdownMenu = _G[UIDROPDOWNMENU_INIT_MENU]
	local showEntry
	
	for index, value in ipairs(UnitPopupMenus[dropdownMenu.which]) do
		if value == "GW_LISTALTS" then
			-- only show this dropdown menu entry if the character is in the guild
			if GuildRosterTable[dropdownMenu.name] then
				showEntry = 1
			else
				showEntry = 0
			end
			UnitPopupShown[UIDROPDOWNMENU_MENU_LEVEL][index] = showEntry
			break
		end
	end
end)

-- add a "List alts" entry to unit popup menus
UnitPopupButtons["GW_LISTALTS"] = { text = "List alts", dist = 0, func = UnitPopup_OnClick}
tinsert(UnitPopupMenus["FRIEND"], #UnitPopupMenus["FRIEND"] - 1, "GW_LISTALTS")


local frame = CreateFrame("Frame")
frame:SetScript("OnEvent", function(self, event, ...)
	if event == "ADDON_LOADED" then
		if select(1, ...) == "GuildWho" then
			GuildWhoData = GuildWhoData or {
				note = "officer",
			}
			frame:RegisterEvent("GUILD_ROSTER_UPDATE")
		end
	elseif IsInGuild() then
		GuildWho_Debug(" ----- ")
		GuildWho_Debug("Event fired: "..event)
	else
		-- you aren't in a guild, return
		GuildWho_Debug("Not in a guild.")
		return
	end

	if event == "CHAT_MSG_SYSTEM" then
		local patterns = {
			"^"..ERR_FRIEND_ONLINE_SS:format("([^|]+)", ""):gsub("%.", "%%."):gsub("%[%]", ".+").."$",
			"^"..ERR_FRIEND_OFFLINE_S:format("(%S+)"):gsub("%.", "%%.").."$",
			"^"..ERR_GUILD_JOIN_S:format("(%S+)"):gsub("%.", "%%.").."$",
			"^"..ERR_GUILD_LEAVE_S:format("(%S+)"):gsub("%.", "%%.").."$",
			"^"..ERR_GUILD_REMOVE_SS:format("(%S+)", "(%S+)"):gsub("%.", "%%.").."$",
		}
		
		-- match the CHAT_MSG_SYSTEM message against global strings
		for i = 1, #patterns do
			local name = string.match(select(1, ...), patterns[i])
			if name and (GuildRosterTable and GuildRosterTable[name] or (i >= 3)) then
				-- GuildWho_Debug("patterns["..i.."] = "..patterns[i])
				if i >= 4 then
					-- message matches a character leaving or being kicked out of the guild, remove from guild roster table
					GuildRosterTable[name] = nil
				elseif i == 2 then
					-- someone going offline
					GuildRosterTable[name]["online"] = nil
				else
					if i == 3 then
						-- someone joins the guild, add to guild roster table
						GuildRosterTable[name] = {}
					end
					GuildRosterTable[name]["online"] = 1
				end
				if i <= 3 then
					GuildWho_Debug("GuildRosterTable["..name.."] = {")
					GuildWho_Debug("    online = "..tostring(GuildRosterTable[name]["online"]))
					GuildWho_Debug("    note = "..tostring(GuildRosterTable[name]["note"]))
					GuildWho_Debug("}")
				else
					GuildWho_Debug("GuildRosterTable["..name.."] = "..tostring(GuildRosterTable[name]))
				end
				-- try to update the guild roster
				GuildRoster()
				break
			end
		end
	elseif event == "GUILD_ROSTER_UPDATE" then
		if delayedCall then
			GuildWho_Debug("Delayed call")
			_G["GuildWho_SlashHandler"..delayedCall[1]](delayedCall[2])
			delayedCall = nil
		end
		if not rosterCooldown then
			frame:SetScript("OnUpdate", GuildWho_OnUpdate)
			GuildWho_Debug("Roster cooldown start")
			rosterCooldownStart = GetTime()
			rosterCooldown = true
		end
		GuildWho_UpdateGuildRoster()
	elseif event == "PLAYER_LOGIN" then
		-- force update guild roster on login
		GuildRoster()
		GuildWho_Debug("GuildRoster()")
	end
end)
frame:RegisterEvent("ADDON_LOADED")
frame:RegisterEvent("CHAT_MSG_SYSTEM")
frame:RegisterEvent("PLAYER_LOGIN")